@@ -9,7 +9,7 @@ import numpy as np
 import matplotlib.pyplot as plt
 from iblutil.util import range_str
 
-import ibllib.dsp.utils as dsp
+import neurodsp.utils as dsp
 from ibllib.plots import squares, vertical_lines
 from ibllib.io.video import assert_valid_label, VideoStreamer
 from iblutil.numerical import within_ranges
