@@ -12,7 +12,7 @@ import pandas as pd
 import scipy.signal
 import matplotlib.pyplot as plt
 
-from ibllib.dsp import voltage
+from neurodsp import voltage
 from ibllib.plots.snapshot import ReportSnapshotProbe, ReportSnapshot
 from one.api import ONE
 import one.alf.io as alfio
@@ -87,10 +87,10 @@ class BehaviourPlots(ReportSnapshot):
                  }
 
     def __init__(self, eid, session_path=None, one=None, **kwargs):
-        self.one = one or ONE()
+        self.one = one
         self.eid = eid
         self.session_path = session_path or self.one.eid2path(self.eid)
-        super(BehaviourPlots, self).__init__(self.session_path, self.eid,
+        super(BehaviourPlots, self).__init__(self.session_path, self.eid, one=self.one,
                                              **kwargs)
         self.output_directory = self.session_path.joinpath('snapshot', 'behaviour')
         self.output_directory.mkdir(exist_ok=True, parents=True)
@@ -478,16 +478,16 @@ def ephys_bad_channels(raw, fs, channel_labels, channel_features, channels=None,
     if plot_backend == 'matplotlib':
         _, axs = plt.subplots(1, 2, gridspec_kw={'width_ratios': [.95, .05]}, figsize=(16, 9))
         eqcs.append(Density(butt, fs=fs, taxis=1, ax=axs[0], title='highpass', vmin=eqc_levels[0], vmax=eqc_levels[1],
-                            cmap='Greys'))
+                            cmap='Greys-r'))
 
         if destripe:
             dest = voltage.destripe(raw, fs=fs, channel_labels=channel_labels)
             _, axs = plt.subplots(1, 2, gridspec_kw={'width_ratios': [.95, .05]}, figsize=(16, 9))
             eqcs.append(Density(dest, fs=fs, taxis=1, ax=axs[0], title='destripe', vmin=eqc_levels[0], vmax=eqc_levels[1],
-                                cmap='Greys'))
+                                cmap='Greys-r'))
             _, axs = plt.subplots(1, 2, gridspec_kw={'width_ratios': [.95, .05]}, figsize=(16, 9))
             eqcs.append(Density((butt - dest), fs=fs, taxis=1, ax=axs[0], title='difference', vmin=eqc_levels[0],
-                                vmax=eqc_levels[1], cmap='Greys'))
+                                vmax=eqc_levels[1], cmap='Greys-r'))
 
         for eqc in eqcs:
             y, x = np.meshgrid(ioutside, np.linspace(0, rl * 1e3, 500))
@@ -599,7 +599,7 @@ def raw_destripe(raw, fs, t0, i_plt, n_plt,
     '''
 
     # Import
-    from ibllib.dsp import voltage
+    from neurodsp import voltage
     from ibllib.plots import Density
 
     # Init fig
@@ -618,7 +618,7 @@ def raw_destripe(raw, fs, t0, i_plt, n_plt,
         Tplot = Xs.shape[1] / fs
 
         # PLOT RAW DATA
-        d = Density(-Xs, fs=fs, taxis=1, ax=axs[i_plt], vmin=MIN_X, vmax=MAX_X, cmap='Greys') # noqa
+        d = Density(-Xs, fs=fs, taxis=1, ax=axs[i_plt], vmin=MIN_X, vmax=MAX_X, cmap='Greys-r') # noqa
         axs[i_plt].set_ylabel('')
         axs[i_plt].set_xlim((0, Tplot * 1e3))
         axs[i_plt].set_ylim((0, nc))
