@@ -9,12 +9,13 @@ import random
 import string
 
 from one.api import ONE
+from neuropixel import SITES_COORDINATES
+
 from ibllib.tests import TEST_DB
 from ibllib.atlas import AllenAtlas
 from ibllib.pipes.misc import create_alyx_probe_insertions
 from ibllib.qc.alignment_qc import AlignmentQC
 from ibllib.pipes.histology import register_track
-from ibllib.ephys.neuropixel import SITES_COORDINATES
 
 
 EPHYS_SESSION = 'b1c968ad-4874-468d-b2e4-5ffa9b9964e9'
