@@ -28,6 +28,9 @@ class TestBrainRegions(unittest.TestCase):
     def setUpClass(self):
         self.brs = BrainRegions()
 
+    def test_rgba(self):
+        assert self.brs.rgba.shape == (self.brs.rgb.shape[0], 4)
+
     def test_get(self):
         ctx = self.brs.get(688)
         self.assertTrue(len(ctx.acronym) == 1 and ctx.acronym == 'CTX')
@@ -41,6 +44,23 @@ class TestBrainRegions(unittest.TestCase):
         d = self.brs.descendants(ids=leaves['id'])
         self.assertTrue(np.all(np.sort(leaves['id']) == np.sort(d['id'])))
 
+    def test_ancestors_descendants_indices(self):
+        br = self.brs
+        tpath = np.array([997, 8, 567, 688, 695, 315, 453, 12993])
+        # /997/8/567/688/695/315/453/12993/
+        # check ancestors
+        ancs = br.ancestors(12993)
+        assert np.all(ancs.id == tpath)
+        # check ancestors with indices
+        ancs, inds = br.ancestors(12993, return_indices=True)
+        assert np.all(ancs.id == tpath)
+        # check descendants with indices
+        desdc, inds = br.descendants(12993, return_indices=True)
+        assert(inds == np.where(br.id == 12993))
+        # check full subtree
+        chemin = br.subtree(453)
+        assert np.all(np.sort(chemin.id) == np.unique(np.r_[br.descendants(453).id, br.ancestors(453).id]))
+
     def test_mappings_lateralized(self):
         # the mapping assigns all non found regions to root (1:997), except for the void (0:0)
         # here we're looking at the retina (1327:304325711), so we expect 1327 at index 1327
@@ -252,6 +272,18 @@ class TestBrainRegions(unittest.TestCase):
         assert np.array_equal(acronnyms_ordered, expected_acronyms)
         assert np.array_equal(values_ordered, expected_values)
 
+    def test_argument_parser(self):
+        acronyms = ['AUDp1', 'AUDpo1', 'AUDv1', 'SSp-m1', 'SSp-n1']
+        ids = self.brs.acronym2id(acronyms)
+        assert np.all(self.brs.parse_acronyms_argument(acronyms) == ids)
+        assert np.all(self.brs.parse_acronyms_argument(np.array(acronyms)) == ids)
+        assert np.all(self.brs.parse_acronyms_argument(ids) == ids)
+        assert np.all(self.brs.parse_acronyms_argument(list(ids)) == ids)
+        # makes sure it handles exception
+        with self.assertRaises(AssertionError):
+            self.brs.parse_acronyms_argument(acronyms + ['toto'])
+        assert np.all(self.brs.parse_acronyms_argument(acronyms + ['toto'], mode='clip') == ids)
+
 
 class TestAtlasPlots(unittest.TestCase):
 
@@ -280,11 +312,11 @@ class TestAtlasPlots(unittest.TestCase):
         ax.clear()
 
     def test_horizontal_slice(self):
-        ax = self.ba.plot_hslice(dv_coordinate=0.002)
+        ax = self.ba.plot_hslice(dv_coordinate=-0.002)
         im = ax.get_images()[0]
         assert im.get_array().shape == (self.ba.bc.ny, self.ba.bc.nx)
         ax.clear()
-        ax = self.ba.plot_hslice(dv_coordinate=0.002, volume='annotation')
+        ax = self.ba.plot_hslice(dv_coordinate=-0.002, volume='annotation')
         im = ax.get_images()[0]
         assert im.get_array().shape == (self.ba.bc.ny, self.ba.bc.nx, 3)
         ax.clear()
@@ -321,9 +353,9 @@ class TestAtlasSlicesConversion(unittest.TestCase):
         # tests output shapes
         self.assertTrue(ba.slice(axis=0, coordinate=0).shape == (ny, nz))  # sagittal
         self.assertTrue(ba.slice(axis=1, coordinate=0).shape == (nx, nz))  # coronal
-        self.assertTrue(ba.slice(axis=2, coordinate=.002).shape == (ny, nx))  # horizontal
+        self.assertTrue(ba.slice(axis=2, coordinate=-.002).shape == (ny, nx))  # horizontal
         # tests out of bound
-        with self.assertRaises(IndexError):
+        with self.assertRaises(ValueError):
             ba.slice(axis=1, coordinate=123)
         self.assertTrue(ba.slice(axis=1, coordinate=21, mode='clip').shape == (nx, nz))
         """
@@ -521,9 +553,9 @@ class TestsCoordinatesSimples(unittest.TestCase):
         self.assertTrue(bc.ny == 7)
         self.assertTrue(bc.nz == 8)
         # test array functions
-        in_out = [([6, 7, 8], np.array([6, 7, 8])),
-                  (np.array([6, 7, 8]), np.array([6, 7, 8])),
-                  (np.array([[6, 7, 8], [6, 7, 8]]), np.array([[6, 7, 8], [6, 7, 8]])),
+        in_out = [([3, 4, 5], np.array([3, 4, 5])),
+                  (np.array([3, 4, 5]), np.array([3, 4, 5])),
+                  (np.array([[3, 4, 5], [3, 4, 5]]), np.array([[3, 4, 5], [3, 4, 5]])),
                   ]
         for io in in_out:
             self.assertTrue(np.all(bc.xyz2i(io[0]) == io[1]))
